import pygame
import numpy as np
from Temps import *
from Image import transformImage
#from Infrastructure import Infrastructure
listeImage = [transformImage(pygame.image.load('../assets/Plantation/crop'+str(i)+'phase.png'), 120, 120) for i in range(1,5)]
iconStaticSize = (100,100)
"""
@nom = Game :  Class : Super class qui répresente un champs
@param : pas de parametre
@return : void
@Description :
Super class qui répresente un champs
"""
class Champs:
    iconInsecteStatic = pygame.transform.scale(pygame.image.load("../assets/Plantation/leech.png"),(90,90))
    """
    @nom = __init___ :  Méthode Constructor : Constructeur de la classe Champs
    @param : 
    temperature : int : temperature du champs
    humdite : int :  humidité du champs
    tempsAvantMaturation : int : tempsAvantMaturation de la plante
    pos : int : position x et y
    image : string : chemin de l'image 
    @return : void
    @Description :
    Met en place les champs
    """
    def __init__ (self, temperature, humidite, tempsAvantMaturation, pos, image):
        self.pourcentagePlanteFane = 10

        self.temperature = temperature
        self.humidite = humidite
        self.pos = np.array(pos)
        self.image = pygame.image.load(image)
        self.imageSize = np.array(self.image.get_size())
        self.rect = pygame.rect.Rect(self.pos, self.image.get_rect().size)
        self.rectScale = self.rect
        self.police = pygame.font.SysFont("Alef", 22) #pygame.font.get_fonts() -> liste des fonts
        self.imgZoom = pygame.transform.scale(self.image, self.imageSize)
        self.indiceImage = 0
        self.champZoom = pygame.rect.Rect(self.pos,self.imgZoom.get_rect().size)
        self.isInfeste = False
        self.bat = None
        #self.infra = self.setInfrastrcture(self.pos,self.image)


        
        self.descripteurChamps = {
        
        }

        self.imageSurfaceinfo = {
            
        }
        self.setInformation("Occupe" , "Champ non occupé")
        self.setInformation("Nom","Champ Vide ")
        self.setInformation("coutEntretien","90€ coût d'entretien mensuel")
        self.setInformation("fane", "Pourcentage que la plante fanne : ")
        self.setInformation(hash('fane'), str(self.pourcentagePlanteFane)+' %')
        self.setInformation("status", self.image,True)


    """
    @nom = plantationGrandit :  Methode : Change l'image et le status de la plante au tour suivant
    @param :
    @return : void
    @Description :
    Change l'image et le status de la plante au tour suivant
    """
    def plantationGrandit(self):
        self.indiceImage += 1
        if self.indiceImage < 4:
            self.imageChamp = listeImage[self.indiceImage]
        else:
            if(self.indiceImage == 4):
                self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/damage.png'), 120, 120)
            elif(self.indiceImage == 5):
                self.isInfeste = False
                self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/fanne.png'), 120, 120)

    """
    @nom = draw :  Function : Affiche les images et info dans 
    @param :
    @return : void
    @Description :
    Change l'image et le status de la plante au tour suivant
    """
    def draw (self, screen, camera):
        scaleImage = camera.scaleImage(self.image)

        if scaleImage: # != None
            self.imgZoom = scaleImage
            self.rectScale = pygame.rect.Rect(camera.convertLocation(self.pos), self.imgZoom.get_size())
            
        
        
        screen.blit(self.imgZoom, camera.convertLocation(self.pos))
        if hasattr(self,'imageChamp'):
            scaleChamp = camera.scaleImage(self.imageChamp)
            if scaleChamp:
                self.champZoom = scaleChamp
            varlongueur = camera.convertLocation(self.pos)   
            self.rectChampScal = pygame.rect.Rect(varlongueur, self.champZoom.get_size())
            screen.blit(
            self.champZoom, 
            (
                varlongueur[0] + 
                (self.rectScale.w-self.rectChampScal.w)
                /2,
                varlongueur[1] + 
                (self.rectScale.h-self.rectChampScal.h)
                /2)
            ) 

        if(self.isInfeste):
            scaleInfecte = camera.scaleImage(Champs.iconInsecteStatic)
            if scaleInfecte:
                self.infecteZoom = scaleInfecte
            varlongueur = camera.convertLocation(self.pos)   
            self.rectScalInfecte = pygame.rect.Rect(varlongueur, self.infecteZoom.get_size())
            screen.blit(
            self.infecteZoom, 
            (
                varlongueur[0] + 
                (self.rectScale.w-self.rectScalInfecte.w)
                /2,
                varlongueur[1] + 
                (self.rectScale.h-self.rectScalInfecte.h)
                /2)
            )

    """
    @nom = setInformation :  Function : set les informations sur chaque champs
    @param : 
    clef : string : nom de colonne clef du dictionnaire
    valeur : string : nom de colonne valeur du dictionnaire
    isImage : bool : si la valeur et une image
    @return : void
    @Description :
    Change l'image et le status de la plante au tour suivant
    """
    def setInformation(self,clef,valeur,isImage = False):
        self.descripteurChamps[clef] = valeur
        if isImage : 
            self.imageSurfaceinfo[clef] = valeur
        else : 
            self.imageSurfaceinfo[clef] = police.render(str(valeur),True,(0,0,0))
        
    """
    @nom = showinfo :  Function : Affiche les infos sur les champs
    @param : 
    screen : Surface : Surface sur laquelle nous allons écrire
    rectangle : pygame.Rectangle : Surface sur laquelle nous allons écrire
    @return : void
    @Description :
    Change l'image et le status de la plante au tour suivant
    """
    def showInfo(self,screen,rectangle):
        i = 0
        for k,v in self.descripteurChamps.items():
            screen.blit(self.imageSurfaceinfo[k],(rectangle.x,rectangle.y + i))
            i += 30

"""
@nom = Game :  Class : Super class qui répresente un champs
@param : pas de parametre
@return : void
@Description :
Super class qui répresente un champs
"""
class Buisson (Champs):
    def __init__ (self, pos, prix = 10000):
        super().__init__(0, 0, 0, pos, '../assets/Interface/buissonCadenas.png')
        self.prix = prix
        self.text = self.police.render("Champ disponible à l'achat : " + str(self.prix), False, (0,0,0))
        self.zoomText = self.text
    
    def showInfo (self, screen,rectangle):
        screen.blit(self.text,(rectangle.x,rectangle.y))
    
    def buyField (self):
        return TerrainVide(self.pos)


class TerrainVide(Champs):
    def __init__ (self, pos):
        super().__init__(0, 0, 0, pos,'../assets/Plantation/champ.png')
        self.text = self.police.render("Température : " + str(self.temperature), False, (0,0,0))
        self.zoomText = self.text



class Houblon (Champs):
    montantAchat = 250
    montantVente = 600
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/houblon.png'), *iconStaticSize)
    textStatic = police.render('Houblon',False, (0,0,0))
    textDescriptionStatic = police.render("Montant Achat :"+str(montantAchat),False,(0,0,0))
    textDescriptionStatic2 = police.render("Montant Vente :"+str(montantVente),False,(0,0,0))

    def __init__ (self, pos):
        super().__init__(0, 0, 0, pos,'../assets/Plantation/champ.png')
        self.text = self.police.render("Température : " + str(self.temperature), False, (0,0,0))
        self.zoomText = self.text

        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)
        self.descripteurPlante = {
        
        }

        self.imageSurfacePlante = {
            
        }

        super.setInformation("Montant Achat",200)
        super.setInformation("Montant Vente",800)
        print("changement fait")


        
class Ble (Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/wheat.png'), *iconStaticSize)
    textStatic = police.render('Ble',False, (0,0,0))
    textDescriptionStatic = police.render("Ble sa marche bien",False,(0,0,0))
    montantAchat = 250
    montantVente = 800
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/champ.png')
        
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)

class Malt (Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/malt.png'), *iconStaticSize)
    textStatic = police.render('Malt',False, (0,0,0))
    textDescriptionStatic = police.render("Malt sa marche bien",False,(0,0,0))
    montantAchat = 300
    montantVente = 800
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/champ.png')
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)


class Chanvre(Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/weed.png'), *iconStaticSize)
    textStatic = police.render("Chanvre",False, (0,0,0))
    textDescriptionStatic = police.render("Pour des spaces bières",False,(0,0,0))
    montantAchat = 1000
    montantVente = 5000
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets//Plantation/champ.png')
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)


class OrgeHiver(Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/orge.png'), *iconStaticSize)
    textStatic = police.render("Orge hiver",False, (0,0,0))
    textDescriptionStatic = police.render("Orge hiver sa marche bien",False,(0,0,0))
    montantAchat = 300
    montantVente = 800
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/champ.png')
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)


class OrgePrintemps(Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/OrgePrintemps.png'), *iconStaticSize)
    textStatic = police.render("Orge printemps",False, (0,0,0))
    textDescriptionStatic = police.render("Orge printemps sa marche bien",False,(0,0,0))
    montantAchat = 500
    montantVente = 1000
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/champ.png')
        
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)


class Seigle(Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/seigle.png'), *iconStaticSize)
    textStatic = police.render("Seigle",False, (0,0,0))
    textDescriptionStatic = police.render("Seigle sa marche bien",False,(0,0,0))
    montantAchat = 200
    montantVente = 600
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/champ.png')
        self.imageChamp = transformImage(pygame.image.load('../assets/Plantation/crop1phase.png'), 120, 120)


class Pesticide(Champs):
    iconStatic = transformImage(pygame.image.load('../assets/Plantation/pesticide.png'), *iconStaticSize)
    textStatic = police.render("Pesticide",False, (0,0,0))
    textDescriptionStatic = police.render("Pour tuer les insectes",False,(0,0,0))
    montantAchat = 200
    def __init__ (self,pos):
        super().__init__(0,0,0,pos,'../assets/Plantation/pesticide.png')

#TODO : class Bussion,

typePlantation = [Houblon,Ble,Malt,Chanvre,OrgePrintemps,OrgeHiver,Seigle,Pesticide]
    
        